/*****************************************************************************
 * Copyright 2012 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.dbtool.resource;

import com.celum.dbtool.DbException;
import com.celum.dbtool.script.*;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.*;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
 * Script resource take given package where are SQL scripts
 * placed.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class PackageResource extends DbScriptsResource
{
    private final static String JAR_EXTENSION = ".jar!";

    /** package where are SQL Scripts */
    private final String pckgName;

    /** classloader for script loading, default is system classloader */
    private final ClassLoader loader;


    /**
     * Constructor
     * @param pckg package instance
     */
    public PackageResource(Package pckg)
    {
        this.pckgName = pckg.getName();
        this.loader = ClassLoader.getSystemClassLoader();
    }


    /**
     * Constructor
     * @param pckgName package name with '.'
     */
    public PackageResource(String pckgName)
    {
        this.pckgName = pckgName;
        this.loader = ClassLoader.getSystemClassLoader();
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public Collection<DbScript> getSortedScripts(VersionFactory vf)
    {
        try {
            String relPath = pckgName.replace('.', '/');
            List<DbScript> scripts = new LinkedList<DbScript>();
            Enumeration<URL> resources = loader.getResources(relPath);

            //go through all resources
            while (resources.hasMoreElements()) {
                URL resource = resources.nextElement();
                if (resource.getProtocol().compareToIgnoreCase("file") == 0) {
                    scripts = loadFromDir(resource, vf, scripts);
                }
                else if (resource.getProtocol().compareToIgnoreCase("jar") == 0) {
                    scripts = loadFromJar(resource, vf, scripts);
                }
            }

            Collections.sort(scripts);
            return scripts;
        } catch (Exception e) {
            throw new DbException("error loading scripts from package " + pckgName, e);
        }
    }


    /**
     * Method scan directory which represents the JAVA package and load
     * all scripts into 'toList' list.
     */
    private List<DbScript> loadFromDir(URL pckgResource, VersionFactory vf, List<DbScript> toList) throws URISyntaxException
    {
        File pckgDir = new File(pckgResource.toURI());
        File[] files = pckgDir.listFiles();
        for (int i = 0; i < files.length; i++) {
            if (files[i].isFile()) {
                String name = pckgName.replace('.', '/') + "/" + files[i].getName();
                Version v = vf.parse(files[i].getName());
                ScriptType type = ScriptType.getTypeOfFile(files[i].getName());
                if (type != ScriptType.UNSUPPORTED) {
                    toList.add(new ClasspathScript(name, v, type, name));
                }
            }
        }

        return toList;
    }


    /**
     * Method scan package inside JAR and load scripts into 'toList'
     */
    private List<DbScript> loadFromJar(URL pckgResource, VersionFactory vf, List<DbScript> toList) throws URISyntaxException, IOException
    {
        String name = pckgName.replace('.', '/');

        //get and open JAR file
        String resourceName = pckgResource.toString();
        int pos = resourceName.indexOf(JAR_EXTENSION);
        String jarPath = resourceName.substring(4, pos + JAR_EXTENSION.length() - 1);
        JarFile jarFile = new JarFile(new File(new URI(jarPath)));

        //go through JAR entries
        Enumeration<JarEntry> entries = jarFile.entries();
        while (entries.hasMoreElements()) {
            JarEntry entry = entries.nextElement();
            if (!entry.isDirectory()) {
                String entryName = entry.getName();
                if (entryName.startsWith(name)) {
                    //create SQL script
                    ScriptType type = ScriptType.getTypeOfFile(entryName);
                    if (type != ScriptType.UNSUPPORTED) {
                        Version v = vf.parse(entryName.substring(name.length()));
                        toList.add(new ClasspathScript(entryName, v, type, entryName));
                    }
                }
            }
        }

        return toList;
    }
}
