/*****************************************************************************
 * Copyright 2012 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.dbtool.resource;

import com.celum.dbtool.script.DbScript;
import com.celum.dbtool.script.Version;
import com.celum.dbtool.script.VersionFactory;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/**
 * Decorator for {@link DbScriptsResource resource} that is responsible for
 * filtering SQL scripts based on script version.
 *
 * The filter could have various strategies when scripts are filtered
 * when script version is lower as value, or bigger. The concrete
 * filter is constructed via filter() method and rule.
 *
 * example:
 * <code>
 *     SqlScriptResource res = ...;
 *     SqlScriptResource filteredRes = VersionFilter.filter(res).largerThan(new LongVersion(10));
 *     Collection<DbScript> scripts = filteredRes.getSortedScripts();
 * </code>
 * Example above returns sorted collection of scripts and only these scripts which version is after nr. 10.
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class VersionFilter extends DbScriptsResource
{
	/** referring to real sql script resource */
	private final DbScriptsResource decoratedResource;


    /**
     * hidden constructor, filter is constructed via filter() method.
     */
    private VersionFilter(DbScriptsResource decoratedResource)
    {
        this.decoratedResource = decoratedResource;
    }


    /**
     * factory method that construct builder which is responsible for
     * concrete VersionFilter impl.
     */
    public static Builder filter(DbScriptsResource resource)
    {
        return new Builder(resource);
    }


    /**
     * Builder is responsible for construction of VersionFilter with
     * concrete filter rule.
     */
    public static class Builder
    {
        private DbScriptsResource decoratedResource;

        private Builder(DbScriptsResource decoratedResource)
        {
            this.decoratedResource = decoratedResource;
        }

        public VersionFilter largerThan(Version value)
        {
            return new LargerThan(decoratedResource, value);
        }

        public VersionFilter smallerOrEqualsTo(Version value)
        {
            return new SmallerOrEqualsTo(decoratedResource, value);
        }

        public VersionFilter inBetween(Version min, Version max)
        {
            return new InBetween(decoratedResource, min, max);
        }
    }


    /**
     * Larger than filter impl.
     */
    private static class LargerThan extends VersionFilter
    {
        private final Version value;

        /**
         * hidden constructor
         */
        private LargerThan(DbScriptsResource decoratedResource, Version value) {
            super(decoratedResource);
            this.value = value;
        }

        /**
         * filter rule for 'larger than'.
         */
        @Override
        protected boolean filter(Version scriptVersion) {
            return scriptVersion.compareTo(value) > 0;
        }
    }


    /**
     * smaller or equals to filter impl.
     */
    private static class SmallerOrEqualsTo extends VersionFilter
    {
        private final Version value;

        /**
         * hidden constructor
         */
        private SmallerOrEqualsTo(DbScriptsResource decoratedResource, Version value)
        {
            super(decoratedResource);
            this.value = value;
        }

        /**
         * filter rule for 'smaller of equals to'.
         */
        @Override
        protected boolean filter(Version scriptVersion) {
            return scriptVersion.compareTo(value) > 0;
        }
    }


    /**
     * In between filter impl.
     */
    private static class InBetween extends VersionFilter
    {
        private final Version min;
        private final Version max;

        /**
         * hidden constructor
         */
        private InBetween(DbScriptsResource decoratedResource, Version min, Version max)
        {
            super(decoratedResource);
            this.min = min;
            this.max = max;
        }

        /**
         * filter rule 'in between'
         */
        @Override
        protected boolean filter(Version scriptVersion) {
            return (scriptVersion.compareTo(min) > 0) && (scriptVersion.compareTo(max) < 0);
        }
    }


    /**
     * Method get DB version and returns only these scripts which have
     * bigger version number
     */
	@Override
	final public Collection<DbScript> getSortedScripts(VersionFactory vf)
	{
		Collection<DbScript> scripts = decoratedResource.getSortedScripts(vf);
		scripts = filterScriptsByVersion(scripts);
		return scripts;
	}


    /**
     * method filters the scripts by version.
     */
	final Collection<DbScript> filterScriptsByVersion(Collection<DbScript> scripts)
	{
		List<DbScript> out = new LinkedList<DbScript>();
		for (DbScript script : scripts) {
            boolean allowed = filter(script.getVersion());
            if (allowed) {
                out.add(script);
            }
        }

		Collections.sort(out);
		return out;
	}


    /**
     * filter method is abstract because there are several rules like <, >, =, or is in range
     *
     * @return true when script version goes though filter, false when script will be ignored
     */
    protected abstract boolean filter(Version scriptVersion);

}
