/*****************************************************************************
 * Copyright 2012 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.dbtool.script;

import java.io.Reader;

/**
 * Abstraction that representing DB script. There are various implementations
 * of scripts depends on source. Also script could be SQL or Groovy depends
 * on his {@link ScriptType type}.
 *
 * @see com.celum.dbtool.resource.DbScriptsResource
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public abstract class DbScript implements Comparable<DbScript>
{
    /** name of script, e.g.: file name etc */
	private final String name;

    /** version of script */
    private final Version version;

    /** script type */
    private final ScriptType type;


    /**
     * Constructor
     *
     * @param name name of the script (e.g: file name etc)
     */
	public DbScript(String name, Version v, ScriptType type)
	{
		this.name = name;
        this.version = v;
        this.type = type;
	}


    /**
     * Here you will implement code that returns reader for
     * your script impl.
     *
     * @see StringScript
     * @see FileScript
     *
     * @return script reader
     */
    public abstract Reader getScriptReader();


    /**
     * comparing is based on versions all the time.
     * If you want affect the sorting of scripts, change
     * the {@link Version} compare method
     */
    @Override
    public final int compareTo(DbScript script)
    {
        return this.version.compareTo(script.version);
    }


    /**
     * returns script name
     * @return
     */
    public final String getName()
	{
		return name;
	}


    /**
     * returns script version
     * @return
     */
    public final Version getVersion()
    {
        return version;
    }


    /**
     * returns type of script
     * @return
     */
    public ScriptType getType() {
        return type;
    }
}
