/*****************************************************************************
 * Copyright 2012 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.dbtool.sql;

import com.celum.dbtool.installer.IOStreamException;
import org.antlr.v4.runtime.*;
import org.apache.commons.io.input.ReaderInputStream;

import java.io.*;

/**
 * The loader goes through SQL Script and do parsing of each query which
 * is then processed by current {@link SqlScriptStrategy strategy} impl.
 *
 * Because loader has DSL based API, here are examples how to use it.
 *
 * example of load queries into list:
 * <code>
 *     InputStream is = ..;
 *     List<String> queries =
 *             SqlScriptLoader.loadScriptFromStream(is)
 *                            .viaStrategy(new ListStrategy())
 *                            .andReturnListOfQueries();
 * </code>
 *
 * example of your own strategy impl:
 * <code>
 *     InputStream is = ..;
 *     SqlScriptLoader
 *         .loadScriptFromStream(is)
 *         .viaStrategy( new SqlScriptStrategy() {
 *                 public void query(String sql) {
 *                     System.out.println("sql:" + sql);
 *                 }
 *         });
 * </code>
 *
 * @see SqlScriptStrategy
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class SqlScriptLoader {

    /** carries the input stream of script */
    private final CharStream script;

    /**
     * Hidden constructor
     * @param script
     */
    private SqlScriptLoader(CharStream script)
    {
        this.script = script;
    }


    /**
     * Creates loader for script stream.
     */
    public static SqlScriptLoader loadScriptFromStream(InputStream is)
    {
        try {
            return new SqlScriptLoader(new ANTLRInputStream(is));
        } catch (IOException e) {
            throw new IOStreamException(e);
        }
    }


    /**
     * Creates loader for script reader.
     */
    public static SqlScriptLoader loadScriptFromReader(Reader reader)
    {
        return loadScriptFromStream(new ReaderInputStream(reader));
    }


    /**
     * Loads script from file source.
     */
    public static SqlScriptLoader loadScriptFromFile(File f)
    {
        try {
            return new SqlScriptLoader(new ANTLRFileStream(f.getAbsolutePath()));
        } catch (IOException e) {
            throw new IOStreamException(e);
        }
    }


    /**
     * Loads script as text.
     *
     * @param scriptText
     * @return
     */
    public static SqlScriptLoader loadScriptAsText(String scriptText)
    {
        return loadScriptFromReader(new StringReader(scriptText));
    }


    /**
     * Method load and parse SQL script into sql queries and call
     * {@link SqlScriptStrategy strategy} 'query()' method for each query.
     *
     * @param strategy determines how the Query will be processed
     *
     * @return method return the same strategy you fill in because DSL reasons
     *         and strategy could contain the results of loading.
     */
    public <R extends SqlScriptStrategy> R viaStrategy(R strategy)
    {
        //create and prepare lexer, parser and grammar-listener which store the result
        SqlGrammarLexer lexer = new SqlGrammarLexer(script);
        SqlGrammarParser parser = new SqlGrammarParser(new CommonTokenStream(lexer));
        ParserGrammarListener pgl = new ParserGrammarListener(strategy);
        parser.addParseListener(pgl);

        //parsing the SQL script
        parser.sql();
        return strategy;
    }

}
