/*****************************************************************************
 * Copyright 2012 celum Slovakia s r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *****************************************************************************/
package com.celum.dbtool.sql;

import com.celum.dbtool.DbException;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Map;

/**
 * Decorator is responsible for templating. Means placeholders in SQL will be replaced
 * by real values and modified SQL is then moved to decorated strategy where is
 * processed.
 *
 * example:
 * <code>
 *     Map<String, Object> values = ...;
 *     values.put("user", "mickeymouse");
 *     ...
 *     VelocityDecorator<ListStrategy> strategy = VelocityDecorator.decorate(new ListStrategy(), values);
 *
 *     List<String> sql =
 *         SqlScriptLoader
 *             .loadScriptFromStream(is)
 *             .viaStrategy(strategy)
 *             .fromDecoratedStrategy()
 *             .returnListOfQueries();
 * </code>
 *
 * @author Zdenko Vrabel (zdenko.vrabel@celum.com)
 */
public class VelocityDecorator<T extends SqlScriptStrategy> extends DecoratorScriptStrategy<T>
{
    /** engine does the templating */
    private VelocityEngine engine;

    /** context holds the real values */
    private VelocityContext context;


    /**
     * Use this factory method instead of constructor.
     *
     * @param strategy strategy you wish to decorate
     * @param withValues velocity values/placeholders you wish to replace in SQLs.
     */
    public static <T extends SqlScriptStrategy> VelocityDecorator<T> decorate(T strategy, Map<String, Object> withValues)
    {
        VelocityDecorator<T> decorator = new VelocityDecorator<T>(strategy);
        decorator.initWith(withValues);
        return decorator;
    }


    /**
     * Constructor
     */
    private VelocityDecorator(T decoratedStrategy)
    {
        super(decoratedStrategy);
    }


    /**
     * initialize Velocity engine
     * @param values
     */
    private void initWith(Map<String, Object> values)
    {
        try {
            //initialize velocity
            engine = new VelocityEngine();
            engine.init();

            //create context
            context = new VelocityContext();
            for (Map.Entry<String, Object> e : values.entrySet()) {
                context.put(e.getKey(), e.getValue());
            }
        } catch (Exception e) {
            throw new DbException("Error in velocity decorator:" + e.getMessage(), e);
        }
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public void query(String sql)
    {
        String newSql = doTemplating(sql);
        fromDecoratedStrategy().query(newSql);
    }


    /**
     * method replace velocity placeholders with real
     * values.
     *
     * @param sql original SQL with placeholders
     * @return sql with real values
     */
    private String doTemplating(String sql)
    {
        try {
            StringWriter sw = new StringWriter();
            engine.evaluate(context, sw, "sql", new StringReader(sql));
            sw.flush();
            return sw.toString();
        } catch (IOException e) {
            throw new DbException("Error in velocity decoration:" + e.getMessage());
        }
    }
}
